package jp.jsworld.SpModeMailWatcher;

import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.accessibilityservice.AccessibilityService;
import android.accessibilityservice.AccessibilityServiceInfo;
import android.app.Notification;
import android.app.NotificationManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Vibrator;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.accessibility.AccessibilityEvent;

/**
 * SPモードメールを監視するサービスです。<br>
 * <ol>
 * <li>AccessibilityService クラスを継承して通知ステータスの変更イベントを監視します。
 * <li>ユーザ補助でサービスの実行を許可する必要があります。
 * </ol>
 *
 * @version 1.4.0
 * @author jun shiozaki
 *
 *         【ver. 1.4.0 2012/10/24】<br>
 *         ・スクリーンONで着信音、バイブレーター、LED点滅を停止できる機能を追加<br>
 *         ・通話中に未受信メールを知らせる着信音、バイブレーター、LED点滅をOFFにする機能を追加<br>
 *
 *         【ver. 1.3.2 2012/08/01】<br>
 *         ・バグレポート対策 onStart で NullPointerException で例外が発生しているバグに対応。
 *
 *         【ver. 1.3.0 2012/07/28】<br>
 *         ・画面電源ONで鳴動を停止する機能を追加<br>
 *         ・通知中に操作画面を表示する機能を追加
 *
 *         【ver. 1.2.6 2012/07/08】<br>
 *         ・新着メールを受信してから一定時間未受信メールの通知を無視する機能を追加<br>
 *         ・無効にする時間は SettingReader.getNewMailSpanTime() を使って取得する。<br>
 *         ・着信音の再生時間に１曲再生を追加
 *
 *         【ver. 1.2.4 2012/06/23】<br>
 *         ・LEDの点滅が停止しないバグを修正<br>
 *         ・新着メールの直後に未受信メールを検出しても doCall を呼ばない。<br>
 *         ・鳴動時間を、着信音再生時間とバイブレーター時間に分ける。
 *
 *         【ver. 1.2.3 2012/06/15】<br>
 *         ・toast を表示しないように修正<br>
 *
 *         【ver. 1.2.2 2012/06/06】<br>
 *         ・未受信メールの直後に着信メールを受信した場合にSPモードメールの監視ができなくなるバグを修正<br>
 *
 *         【ver. 1.2.1 2012/06/05】<br>
 *         ・未受信メールの直後に着信メールを受信した場合にLEDの発光が消えないバグを修正<br>
 *
 *         【ver. 1.2.0 2012/06/03】<br>
 *         ・通知をLEDの発光で知らせる処理を追加<br>
 *
 *         【ver. 1.1.0 2012/05/03】<br>
 *         ・使用するメーラーによって未受信メールのテキストを変更するように修正<br>
 */
public class SpModeMailWatchService extends AccessibilityService {
    private static final int NOTIFICATION_TIMEOUT_MILLIS = 1000;
    // private static final int NOTIFICATION_INTERVAL = 20000;
    private static final String LOG_TAG = "SpModeMailWatcher";

    //private static final boolean DEBUG_FLG = true;
    private static final boolean DEBUG_FLG = false;

    // スクリーンONフラグ
    private boolean screenOnFlag = false;

    // 操作画面での停止フラグ
    private boolean ctrlWindowStopFlag = false;

    // 通話状態
    private int callState = TelephonyManager.CALL_STATE_IDLE;

    // 設定読込クラス
    private SettingReader sr = null;

    // スクリーンイベントリスナー
    private BroadcastReceiver mScreenOnListener = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            if (action.equals(Intent.ACTION_SCREEN_ON)) {
                screenOnFlag = true;
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, "********* ACTION_SCREEN_ON *********");

                // スクリーンONでLED の点滅を停止する場合、
                if (sr.getStopLedScreenOn()) {
                    stopLed();
                }

            } else if (action.equals(Intent.ACTION_SCREEN_OFF)) {
                screenOnFlag = false;
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, "********* ACTION_SCREEN_OFF *********");
            }
        }
    };

    // 通話状態のイベントリスナー
    private PhoneStateListener mPhoneStateListener = new PhoneStateListener() {
        public void onCallStateChanged(int state, String number) {

            switch (state) {

            case TelephonyManager.CALL_STATE_RINGING:
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, "********* CALL_STATE_RINGING *********");
                callState = TelephonyManager.CALL_STATE_RINGING;

                // 通話通にLEDの点滅をOFFにする場合、
                if (sr.getOffLedCalling()) {
                    stopLed();
                }
                break;

            case TelephonyManager.CALL_STATE_OFFHOOK:
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, "********* CALL_STATE_OFFHOOK *********");
                callState = TelephonyManager.CALL_STATE_OFFHOOK;

                // 通話通にLEDの点滅をOFFにする場合、
                if (sr.getOffLedCalling()) {
                    stopLed();
                }
                break;

            case TelephonyManager.CALL_STATE_IDLE:
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, "********* CALL_STATE_IDLE *********");
                callState = TelephonyManager.CALL_STATE_IDLE;
                break;
            }
            ;
        }
    };

    // 監視タイマー
    private Timer watchTimer;
    private Date newMailRcvTime = new Date();
    private boolean blinkLed = false;

    @Override
    public void onCreate() {
        if (DEBUG_FLG)
            Log.i(LOG_TAG, "********* onCreate *********");

        newMailRcvTime.setTime(newMailRcvTime.getTime() - (60 * 60 * 60));
        super.onCreate();

        // ACTION_SCREEN_ONを受け取るBroadcastReceiverを登録
        IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_SCREEN_ON);
        filter.addAction(Intent.ACTION_SCREEN_OFF);
        registerReceiver(mScreenOnListener, filter);

        // 通話状態を受け取るリスナーを設定
        TelephonyManager telephonyManager = (TelephonyManager) getSystemService(TELEPHONY_SERVICE);
        telephonyManager.listen(mPhoneStateListener,
                PhoneStateListener.LISTEN_CALL_STATE);

        // 設定読込クラス
        sr = new SettingReader(this);
    }

    /**
     * サービスへ接続時の処理を行います。<br>
     * <ol>
     * <li>AccessibilityService の設定を行います。
     * </ol>
     */
    @Override
    public void onServiceConnected() {
        if (DEBUG_FLG)
            Log.i(LOG_TAG, "********* onServiceConnected *********");

        // AccessibilityService の設定を行う。
        // 通知ステータスの変更時のみイベントを発生させる。
        AccessibilityServiceInfo info = new AccessibilityServiceInfo();
        info.eventTypes = AccessibilityEvent.TYPE_NOTIFICATION_STATE_CHANGED;
        info.feedbackType = AccessibilityServiceInfo.FEEDBACK_GENERIC;
        info.notificationTimeout = NOTIFICATION_TIMEOUT_MILLIS;
        info.flags = AccessibilityServiceInfo.DEFAULT;
        setServiceInfo(info);
    }

    @Override
    public void onStart(Intent intent, int startId) {
        if (DEBUG_FLG)
            Log.i(LOG_TAG, "********* onStart *********");

        super.onStart(intent, startId);

        // 操作画面からの情報を取得する
        if (intent != null) {
            ctrlWindowStopFlag = intent.getBooleanExtra(
                    CtrlWindowActivity.EXID_STOP, false);
        } else {
            ctrlWindowStopFlag = false;
        }

        if (DEBUG_FLG)
            Log.i(LOG_TAG, ">> Stop:" + ctrlWindowStopFlag);

    }

    @Override
    public void onDestroy() {
        if (DEBUG_FLG)
            Log.i(LOG_TAG, "********* onDestroy *********");

        // BroadcastReceiverを登録解除
        unregisterReceiver(mScreenOnListener);

        // 通話状態を受け取るリスナーを解除
        TelephonyManager telephonyManager = (TelephonyManager) getSystemService(TELEPHONY_SERVICE);
        telephonyManager.listen(mPhoneStateListener,
                PhoneStateListener.LISTEN_NONE);

        super.onDestroy();
    }

    @Override
    public void onAccessibilityEvent(AccessibilityEvent event) {
        if (DEBUG_FLG)
            Log.i(LOG_TAG, "********* onAccessibilityEvent *********");

        // イベントタイプを取得します。
        int eventType = event.getEventType();

        // 念のため TYPE_NOTIFICATION_STATE_CHANGED 以外は無視します。
        if (eventType != AccessibilityEvent.TYPE_NOTIFICATION_STATE_CHANGED)
            return;

        // 通知のテキストを取得します。
        String eventText = "";
        try {
            eventText = (String) event.getText().get(0);
        } catch (Exception e) {
            eventText = "";
        }

        if (DEBUG_FLG)
            Log.i(LOG_TAG, "eventText:" + eventText);

        // テキストが無効の場合は終了する。
        if (eventText == null || eventText.equals("")) {
            return;
        }

        SettingReader sr = new SettingReader(this);
        if (eventText.equals(sr.getUnReceivedMailText())) {
            long waitTime = sr.getWaittime() * 1000;

            // 通知受信時間
            long span = new Date().getTime() - newMailRcvTime.getTime();
            if (DEBUG_FLG)
                Log.i(LOG_TAG, ">> Notification Span:" + Long.toString(span));

            // 通知が連続して発生した場合はキャンセルする。
            long newMailSpanTime = sr.getNewMailSpanTime() * 1000;
            if (DEBUG_FLG)
                Log.i(LOG_TAG,
                        ">> New mail span time:"
                                + Long.toString(newMailSpanTime));

            if (span < newMailSpanTime) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Event Cancel");
                return;
            }

            if (waitTime <= 0) {
                // 遅延時間が設定されていない場合はタイマーを使用せずにすぐに鳴動します。
                doCall();
            } else {
                // Toast.makeText(this, "未受信メール通知を受信しました",
                // Toast.LENGTH_LONG).show();

                // すでにタイマーがセットされてい場合はキャンセルする。
                if (this.watchTimer != null) {
                    this.watchTimer.cancel();
                }

                // タイマーにジョブを設定する。
                this.watchTimer = new Timer();
                TimerTask timerTask = new TimerTask() {
                    public void run() {
                        doCall();
                    }
                };

                // タイマーを開始して一定時間待機する。
                watchTimer.schedule(timerTask, waitTime);

                // 処理終了
                return;
            }
        }

        // 新着メールのチェックを行います。
        Pattern p = Pattern.compile("^新着メール");
        Matcher m = p.matcher(eventText);
        if (m.find()) {
            // 受信時間設定
            newMailRcvTime = new Date();

            // 新着メールを受信したらタイマーを停止する。
            // Toast.makeText(this, "新着メール通知を受信しました", Toast.LENGTH_LONG).show();
            if (this.watchTimer != null) {
                this.watchTimer.cancel();
            }

            // LED が発光している場合は停止する。
            stopLed();

            // 処理終了
            return;
        }
    }

    private boolean isCalling = false;
    private Date stTime = null;
    private Timer callTimer = null;
    private Ringtone ringTone = null;
    private Vibrator vibrator = null;

    private boolean soundFlag = true;
    private boolean vibrateFlag = true;
    private boolean ledFlag = true;

    private boolean stopSoundScreenOnFlag = false;
    private boolean stopViblationScreenOnFlag = false;
    // private boolean stopLedScreenOnFlag = false;

    private boolean offSoundCallingFlag = false;
    private boolean offViblationCallingFlag = false;
    private boolean offLedCallingFlag = false;

    private boolean showCtrlWindow = false;

    private long soundPlayTime = 0;
    private long vibrateTime = 0;

    /**
     * 未受信メールの通知を行います。
     */
    public void doCall() {

        // すでに呼び出し中の場合は終了する。
        if (isCalling)
            return;

        // スクリーンONフラグ初期化
        screenOnFlag = false;

        // 操作画面の停止フラグ初期化
        ctrlWindowStopFlag = false;

        try {
            if (DEBUG_FLG)
                Log.i(LOG_TAG, "********* doCall *********");

            // 設定情報取得
            String soundName = sr.getSoundName();
            soundFlag = sr.getNoticeSoundFlag();
            vibrateFlag = sr.getNoticeVibrateFlag();
            ledFlag = sr.getNoticeLedFlag();

            soundPlayTime = sr.getNoticeSoundPlayTime() * 1000;
            vibrateTime = sr.getNoticeVibrateTime() * 1000;
            stopSoundScreenOnFlag = sr.getStopSoundScreenOn();
            stopViblationScreenOnFlag = sr.getStopViblationScreenOn();
            // stopLedScreenOnFlag = sr.getStopLedScreenOn();
            offSoundCallingFlag = sr.getOffSoundCalling();
            offViblationCallingFlag = sr.getOffViblationCalling();
            offLedCallingFlag = sr.getStopLedScreenOn();
            showCtrlWindow = sr.getShowControlWinodow();

            // ########## 通話中の通知停止 ##########
            // 通話中は着信音をOFFにする。
            if (offSoundCallingFlag & (callState != TelephonyManager.CALL_STATE_IDLE)) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> OFF Sound");
                soundFlag = false;
            }

            // 通話中はバイブレーターをOFFにする。
            if (offViblationCallingFlag  & (callState != TelephonyManager.CALL_STATE_IDLE)) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> OFF Viblation");
                vibrateFlag = false;
            }

            // 通話中はLEDをOFFにする。
            if (offLedCallingFlag  & (callState != TelephonyManager.CALL_STATE_IDLE)) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> OFF Led");
                ledFlag = false;
            }

            // 操作画面表示
            if (showCtrlWindow) {
                // 画面の表示時間の取得
                long len = 0;
                if (soundFlag) {
                    if (soundPlayTime == 0) {
                        len = 10 * 1000;
                    } else {
                        len = soundPlayTime;
                    }
                }
                if (vibrateFlag) {
                    if (len < vibrateTime) {
                        len = vibrateTime;
                    }
                }

                if (0 < len) {
                    // 操作画面の表示
                    Intent i = new Intent(this, CtrlWindowActivity.class);
                    i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                    i.addFlags(Intent.FLAG_ACTIVITY_MULTIPLE_TASK);
                    i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    i.putExtra(CtrlWindowActivity.EXID_CALL_TIME, len);
                    this.startActivity(i);
                }
            }

            // 通知 LED 発光
            if (ledFlag) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Start blink Lights:" + sr.getLedColor());

                // 通知バーには表示しません。
                NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
                Notification notification = new Notification();
                notification.flags |= Notification.FLAG_SHOW_LIGHTS;
                notification.ledOnMS = 500;
                notification.ledOffMS = 1500;
                notification.ledARGB = sr.getLedColorRGB(sr.getLedColor());
                notificationManager.notify(R.string.app_name, notification);
                blinkLed = true;
            }

            // 鳴動音の設定
            ringTone = null;
            if (soundFlag) {
                // ※サイレントに設定された場合は null になるので注意。
                if (soundName != null) {
                    Uri uri = Uri.parse(soundName);
                    if (DEBUG_FLG)
                        Log.i(LOG_TAG, "sound URI   ：" + uri.toString());

                    ringTone = RingtoneManager.getRingtone(
                            getApplicationContext(), uri);
                    ringTone.play();
                }
            }

            // バイブレーターの設定
            vibrator = null;
            if (vibrateFlag) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Vibrator:" + sr.getVibratePattern());

                vibrator = sr.getVibrator(sr.getVibratePattern(), 0);
            }

            // 鳴動タイマー設定
            if (DEBUG_FLG)
                Log.i(LOG_TAG, ">> Start Timer");

            stTime = new Date();
            callTimer = new Timer();
            callTimer.schedule(new TimerTask() {
                public void run() {
                    boolean stopSound = false;
                    boolean stopVibrator = false;

                    // 鳴動時間チェック
                    Long span = new Date().getTime() - stTime.getTime();

                    // ########## 着信音再生時間のチェック ##########
                    if (soundFlag) {
                        // 0 の場合は１曲再生
                        if (soundPlayTime == 0) {
                            if (ringTone != null) {
                                // 1曲分の再生が終了している場合は停止する。
                                if (!ringTone.isPlaying()) {
                                    if (DEBUG_FLG)
                                        Log.i(LOG_TAG, ">> End sound");

                                    stopSound = true;
                                }
                            } else {
                                stopSound = true;
                            }
                        } else {
                            if (span < soundPlayTime) {
                                // 着信音の連続再生
                                if (ringTone != null) {
                                    if (!ringTone.isPlaying()) {
                                        ringTone.play();
                                    }
                                }
                            } else {
                                // 設定されている時間を経過している場合は停止する。
                                stopSound = true;
                            }
                        }
                    } else {
                        stopSound = true;
                    }

                    // ########## バイブレーター時間のチェック ##########
                    if (vibrateFlag) {
                        if (span < vibrateTime) {

                        } else {
                            // 設定されている時間を経過している場合は停止する。
                            stopVibrator = true;
                        }
                    } else {
                        stopVibrator = true;
                    }

                    // ########## 画面ONでの通知停止 ##########
                    // スクリーンONで着信音を停止する。
                    if (stopSoundScreenOnFlag & screenOnFlag) {
                        stopSound = true;
                    }

                    // スクリーンONでバイブレーターを停止する。
                    if (stopViblationScreenOnFlag & screenOnFlag) {
                        stopVibrator = true;
                    }

                    // ########## 通話中の通知停止 ##########
                    // 通話中は着信音をOFFにする。
                    if (offSoundCallingFlag
                            & (callState != TelephonyManager.CALL_STATE_IDLE)) {
                        stopSound = true;
                    }

                    // 通話中はバイブレーターをOFFにする。
                    if (offViblationCallingFlag
                            & (callState != TelephonyManager.CALL_STATE_IDLE)) {
                        stopVibrator = true;
                    }

                    // ########## 操作画面での鳴動停止 ##########
                    if (showCtrlWindow) {
                        // 画面電源ONで着信音とバイブレーターを停止する。
                        if (ctrlWindowStopFlag) {
                            stopSound = true;
                            stopVibrator = true;
                        }
                    }

                    // ########## 着信音とバイブレーターの停止 ##########
                    // 着信音の停止
                    if (stopSound) {
                        if (ringTone != null) {
                            if (DEBUG_FLG)
                                Log.i(LOG_TAG, ">> Sound Stop.");

                            ringTone.stop();
                            ringTone = null;
                        }
                    }

                    // バイブレーターの停止
                    if (stopVibrator) {
                        // バイブレーターの停止
                        if (vibrator != null) {
                            if (DEBUG_FLG)
                                Log.i(LOG_TAG, ">> Vibrator Stop.");

                            vibrator.cancel();
                            vibrator = null;
                        }
                    }

                    if (stopSound & stopVibrator) {
                        // タイマー停止
                        callTimer.cancel();

                        if (DEBUG_FLG)
                            Log.i(LOG_TAG, ">> Stop Timer");
                    }
                }
            }, 500, 500);

        } catch (Exception e) {
            e.printStackTrace();
        }

        isCalling = false;
    }

    /**
     * LED の点滅を停止します。
     */
    private void stopLed() {
        // LED が発光している場合はLEDを停止する。
        if (blinkLed) {
            if (DEBUG_FLG)
                Log.i(LOG_TAG, ">> Stop blink Lights");

            NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
            Notification notification = new Notification();
            notificationManager.notify(R.string.app_name, notification);
            blinkLed = false;
        }
    }

    @Override
    public void onInterrupt() {
    }
}
